from py4j.java_gateway import JavaGateway
from .utils import get_default_data
import os
from py4j.java_collections import ListConverter


def generate_atom_sel_obj(atom_name_list=[], group_name_list=[], charged=False, element_name_list=[], group_type=None):
    return SPARK_UTILS.generateAtomSelectObject(
        ListConverter().convert(atom_name_list, gateway._gateway_client),
        ListConverter().convert(group_name_list, gateway._gateway_client), charged,
        ListConverter().convert(element_name_list, gateway._gateway_client), group_type
    )
def setup(path_to_file=None):
    """Setup the java gateway and initialise the data"""
    global gateway
    gateway = JavaGateway()
    global SPARK_UTILS
    global BIOJAVA_SPARK_UTILS
    SPARK_UTILS = gateway.entry_point.getSparkUtils()
    BIOJAVA_SPARK_UTILS = gateway.entry_point.getBiojavaSparkUtils()
    if path_to_file == None:
        path_to_file = get_default_data()
    if not os.path.isdir(path_to_file):
        SPARK_UTILS.downloadPdb(path_to_file)
    SPARK_UTILS.updatePdb(path_to_file)
    # Now set up the RDD
    the_pdb = SPARK_UTILS.getStructureObj(os.path.join(path_to_file,"full"))
    return StructureDataRDD(the_pdb),SPARK_UTILS,gateway

def get_subset(subset):
    if type(subset)==list:
        subset = ListConverter().convert(subset, gateway._gateway_client)
        return StructureDataRDD(SPARK_UTILS.selectSubset(os.path.join(get_default_data(),"full"),subset))
    elif type(subset)==float:
        return StructureDataRDD(SPARK_UTILS.selectSubset(os.path.join(get_default_data(), "full"), subset))

class StructureDataRDD():
    """A class to hold, filter and process an archive of PDB entries"""

    def __init__(self, input_rdd):
        """Initialiser takes an RDD"""
        self.structureDataRdd = input_rdd
        self.property_dict = {"R-FREE": self.structureDataRdd.filterRfree,
                              "RESOLUTION": self.structureDataRdd.filterResolution}

    def filter(self, property, max, min=None):
        """Allows the user to filter on a given property"""
        if property not in self.property_dict:
            return False
        return self.property_dict[property](max,min)

    def quick_size(self):
        return self.structureDataRdd.quickSize()

    def get_properties(self):
        """Give a list of properties available for filtering"""
        return self.property_dict.keys()

    def get_atom_dataset(self):
        return AtomDataset(self.structureDataRdd.getAtoms())


    def get_atoms(self, atom_name_list=[], group_name_list=[], charged=False, element_name_list=[], group_type=None):
        """Give a list of atoms - with an optional filter"""
        return AtomDataRDD(BIOJAVA_SPARK_UTILS.findAtoms(self.structureDataRdd,
                                                         generate_atom_sel_obj(atom_name_list,group_name_list,charged,element_name_list,group_type)))

    def get_contacts(self, atom_name_list=[], group_name_list=[], charged=False, element_name_list=[], group_type=None, distance=5.0
                     ,atom_name_list_two=[], group_name_list_two=[], charged_two=False, element_name_list_two=[], group_type_two=None,):
        """Give a list of contacts - with optional filters"""

        if atom_name_list_two or group_name_list_two or element_name_list_two or group_type_two or charged_two:
            return ContactRDD(BIOJAVA_SPARK_UTILS.findContacts(self.structureDataRdd,
                                                               generate_atom_sel_obj(atom_name_list, group_name_list,
                                                                                     charged, element_name_list,
                                                                                     group_type),
                                                               generate_atom_sel_obj(atom_name_list_two, group_name_list_two,
                                                                                     charged_two, element_name_list_two,group_type_two)
                                                               , distance))
        else:
            return ContactRDD(BIOJAVA_SPARK_UTILS.findContacts(self.structureDataRdd,
                                                           generate_atom_sel_obj(atom_name_list, group_name_list,
                                                                                 charged, element_name_list,
                                                                                 group_type),distance))

    def get_reduced(self):
        return StructureDataRDD(self.structureDataRdd.getReduced())

    def get_groups(self, input_filter=None):
        """Give a list of groups - with optional filters"""
        """A cool example here - could be get all the ligands and return them
        as RDKit molecules. Then we have the full searching capabilities of Spark
        and the RDKit functionality to build Pharmacophores and stuff."""
        if not input_filter:
            return GroupDataRDD(self.structureDataRdd)
        else:
            print "Not setup yet to take filters"
            return None

    def get_chains(self, input_filter=None):
        """Give a list of chains - with optional filters"""
        pass

    def get_calpha(self):
        return SegmentDataRDD(self.structureDataRdd.getCalpha())


    def get_fragments(self, frag_length):
        return SegmentDataRDD(self.structureDataRdd.getFragments(frag_length))


class SegmentDataRDD():

    def __init__(self, input_rdd):
        self.segment_data_rdd = input_rdd

    def filter_length(self, min=None, max=None):
        if min and not max:
            return SegmentDataRDD(self.segment_data_rdd.filterMinLength(min))
        elif max and not min:
            return SegmentDataRDD(self.segment_data_rdd.filterMaxLength(max))
        elif max and min:
            return SegmentDataRDD(self.segment_data_rdd.filterLength(min, max))
        else:
            print("Must enter min or max")
            return SegmentDataRDD(self.segment_data_rdd)

    def get_len_dist(self):
        return self.segment_data_rdd.getLengthDist()

    def get_non_redundant_set(self, similarity):
        return SegmentDataRDD(self.segment_data_rdd.findNonRedundantSet(similarity))

    def group_by_sequence(self):
        return SegmentClusterRDD(self.segment_data_rdd.groupBySequence())


class SegmentClusterRDD():
    def __init__(self, input_rdd):
        self.segment_cluster_rdd = input_rdd


class Results():

    def __init__(self, input_results):
        self.results = input_results


    def show(self, num_lines=0):
        if num_lines:
            self.results.show(num_lines)
        else:
            self.results.show()

    def sql_stream(self, sql_query, file_path="temp"):
        return Results(self.results.streamRes(sql_query,file_path))




class AtomDataset():

    def __init__(self, input_dataset):
        self.atom_dataset = input_dataset
        self.table_name = self.atom_dataset.getTableName()

    def sql(self, sql_query, view_name="temp"):
        return Results(self.atom_dataset.sql(sql_query,view_name))

    def sql_stream(self, sql_query, file_path="temp"):
        return Results(self.atom_dataset.streamRes(sql_query,file_path))

    def show(self,num_lines=0):
        if not num_lines:
            self.atom_dataset.show()
        else:
            self.atom_dataset.show(num_lines)

    def cache(self):
        self.atom_dataset.cache()

class AtomDataRDD():
    """A class to hold a list of atoms in"""

    def __init__(self, input_rdd):
        self.atom_data_rdd = input_rdd

    def count_by_atom(self):
        return self.atom_data_rdd.countByAtomName()

    def count_by_element(self):
        return self.atom_data_rdd.countByElement()

    def count_by_group_atom_name(self):
        return self.atom_data_rdd.countByGroupAtomName()

class ContactRDD():
    """A class to hold a list of contacts in - contacts can be POINT-POINT, VECTOR-POINT, VECTOR-VECTOR, PLANE-PLANE """

    def __init__(self, input_rdd):
        self.structure_data_rdd = input_rdd

    def get_distance_dist_of_atom_ints(self, atom_one, atom_two):
        return self.structure_data_rdd.getDistanceDistOfAtomInts(atom_one, atom_two)

    def count_inter_group_contacts(self, group_name_one, group_name_two):
        return self.structure_data_rdd.countInterGroupContacts(group_name_one, group_name_two)

    def get_all_inter_group_contacts(self):
        return self.structure_data_rdd.getAllInterGroupContacts()

    def count_inter_group_contacts(self, group_name_one, group_name_two):
        return self.structure_data_rdd.countInterGroupContacts(group_name_one, group_name_two)

    def count_inter_element_contacts(self, element_name_one, element_name_two):
        return self.structure_data_rdd.countInterElementContacts(element_name_one, element_name_two)

    def count_inter_atom_contacts(self, atom_name_one, atom_name_two):
        return self.structure_data_rdd.countInterAtomContacts(atom_name_one, atom_name_two)

    def get_all_inter_atom_name_contacts(self):
        return self.structure_data_rdd.getAllInterAtomNameContacts()

    def get_all_inter_element_name_contacts(self):
        return self.structure_data_rdd.getAllInterAtomElementContacts()

    def filter_element_group_contacts(self, element_name, group_name):
        return self.structure_data_rdd.filterElementGroupContacts(group_name, element_name)

    def filter_element_element_contacts(self, element_name_one, element_name_two):
        return self.structure_data_rdd.filterElementElementContacts(element_name_one, element_name_two)

    def get_pdb_ids(self):
        return self.structure_data_rdd.getPdbIds()

    def get_group_ids(self):
        return self.structure_data_rdd.getGroupIds()

    def get_atoms(self):
        return AtomDataRDD(self.structure_data_rdd.getAtoms())

class GroupDataRDD():
    """A class to hold a list of groups in"""

    def __init__(self, input_rdd):
        self.groupDataRdd = input_rdd

    def count_by_group_name(self):
        return self.groupDataRdd.countByGroupName()

    def get_atoms(self):
        return self.groupDataRdd.getAtoms()

class ChainDataRDD():
    """A class to hold a list of chains in"""

    def __init__(self, input_rdd):
        self.chain_data_rdd = input_rdd

    def get_polymer_chains(self):
        return ChainDataRDD(self.chain_data_rdd.getPolymerChains())

    def get_non_polymer_chains(self):
        return ChainDataRDD(self.chain_data_rdd.getNonPolymerChains())

    def get_water_chains(self):
        return ChainDataRDD(self.chain_data_rdd.getWaterChains())

    def get_groups(self):
        return GroupDataRDD(self.chain_data_rdd.getGroups())
